!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!  Module :  s i c o _ v a r s _ m
!
!> @file
!!
!! Declarations of global variables for SICOPOLIS (for the ANT domain).
!!
!! @section Copyright
!!
!! Copyright 2009-2019 Ralf Greve
!!
!! @section License
!!
!! This file is part of SICOPOLIS.
!!
!! SICOPOLIS is free software: you can redistribute it and/or modify
!! it under the terms of the GNU General Public License as published by
!! the Free Software Foundation, either version 3 of the License, or
!! (at your option) any later version.
!!
!! SICOPOLIS is distributed in the hope that it will be useful,
!! but WITHOUT ANY WARRANTY; without even the implied warranty of
!! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!! GNU General Public License for more details.
!!
!! You should have received a copy of the GNU General Public License
!! along with SICOPOLIS.  If not, see <http://www.gnu.org/licenses/>.
!<
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

!-------------------------------------------------------------------------------
!> Declarations of global variables for SICOPOLIS (for the ANT domain).
!<------------------------------------------------------------------------------
module sico_vars_m

use sico_types_m

implicit none
save

!> smb_corr_in(j,i): Prescribed SMB correction
   real(dp), dimension(0:JMAX,0:IMAX) :: smb_corr_in

#if (defined(INITMIP_SMB_ANOM_FILE))
!> smb_anom_initmip(j,i): InitMIP anomaly of the accumulation-ablation function
!>                        at the ice surface (surface mass balance)
   real(dp), dimension(0:JMAX,0:IMAX) :: smb_anom_initmip
#endif

#if (defined(INITMIP_BMB_ANOM_FILE))
!> ab_anom_initmip(j,i): InitMIP anomaly of the basal melting rate
!>                       under floating ice
   real(dp), dimension(0:JMAX,0:IMAX) :: ab_anom_initmip
#endif

#if (defined(LARMIP_REGIONS_FILE))
!> n_regions_larmip(j,i): Marker for the different LARMIP regions
!>                        for ice shelf basal melting.
   integer(i1b), dimension(0:JMAX,0:IMAX) :: n_regions_larmip
!> ab_anom_larmip(n): LARMIP anomaly of the basal melting rate
!>                    under floating ice
   real(dp), dimension(0:7) :: ab_anom_larmip
#endif

!> n_sector(j,i): Marker for the different sectors for ice shelf basal melting.
   integer(i1b), dimension(0:JMAX,0:IMAX) :: n_sector

#if (defined(SEDI_SLIDE) \
     && SEDI_SLIDE==2)   /* with sediment sliding */
!> maske_sedi(j,i): Sediment mask.
!>                  1: hard rock,
!>                  7: soft sediment,
!>                  2: ocean.
   integer(i1b), dimension(0:JMAX,0:IMAX) :: maske_sedi
!> r_mask_sedi(j,i): Continuous sediment mask.
!>                   0: hard rock,
!>                   1: soft sediment or ocean,
!>                   between 0 and 1: transition.
   real(dp), dimension(0:JMAX,0:IMAX) :: r_mask_sedi
#endif

end module sico_vars_m
!
