!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!  Module :  s i c o _ v a r s _ m
!
!> @file
!!
!! Declarations of global variables for SICOPOLIS (for the GRL domain).
!!
!! @section Copyright
!!
!! Copyright 2009-2019 Ralf Greve
!!
!! @section License
!!
!! This file is part of SICOPOLIS.
!!
!! SICOPOLIS is free software: you can redistribute it and/or modify
!! it under the terms of the GNU General Public License as published by
!! the Free Software Foundation, either version 3 of the License, or
!! (at your option) any later version.
!!
!! SICOPOLIS is distributed in the hope that it will be useful,
!! but WITHOUT ANY WARRANTY; without even the implied warranty of
!! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!! GNU General Public License for more details.
!!
!! You should have received a copy of the GNU General Public License
!! along with SICOPOLIS.  If not, see <http://www.gnu.org/licenses/>.
!<
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

!-------------------------------------------------------------------------------
!> Declarations of global variables for SICOPOLIS (for the GRL domain).
!<------------------------------------------------------------------------------
module sico_vars_m

use sico_types_m

implicit none
save

!> smb_corr_in(j,i): Prescribed SMB correction
   real(dp), dimension(0:JMAX,0:IMAX) :: smb_corr_in

#if (defined(INITMIP_SMB_ANOM_FILE))
!> smb_anom_initmip(j,i): InitMIP anomaly of the accumulation-ablation function
!>                        at the ice surface (surface mass balance)
   real(dp), dimension(0:JMAX,0:IMAX) :: smb_anom_initmip
#endif

#if (ICE_STREAM==2)   /* with ice streams */
!> maske_sedi(j,i): Sediment mask.
!>                  1: hard rock,
!>                  7: soft sediment,
!>                  2: ocean.
   integer(i1b), dimension(0:JMAX,0:IMAX) :: maske_sedi
#endif

#if (DISC==2)
!> glann_time_min: Minimum time of the data values for the
!>                 global annual temperature anomaly
   integer(i4b) :: glann_time_min
!> glann_time_stp: Time step of the data values for the
!>                 global annual temperature anomaly
   integer(i4b) :: glann_time_stp
!> glann_time_max: Maximum time of the data values for the
!>                 global annual temperature anomaly
   integer(i4b) :: glann_time_max
!> ndata_glann: Number of data values for the global annual temperature anomaly
   integer(i4b) :: ndata_glann
!> dT_glann_CLIMBER(n): Data values for the global annual temperature anomaly
   real(dp), dimension(:), allocatable :: dT_glann_CLIMBER
#endif

end module sico_vars_m
!
